@@ -189,50 +189,52 @@ class Worker:
         time_with_no_tasks = 0
         run_forever = self.config.max_tasks_per_run == 0
         tasks_left = self.config.max_tasks_per_run
-        tasks = []
 
-        try:
-            while tasks_left > 0 or run_forever:
-                if run_forever:
-                    number_of_tasks_to_pull = self.config.tasks_per_transaction
-                else:
-                    number_of_tasks_to_pull = min(
-                        self.config.tasks_per_transaction,
-                        tasks_left,
+        while tasks_left > 0 or run_forever:
+            if run_forever:
+                number_of_tasks_to_pull = self.config.tasks_per_transaction
+            else:
+                number_of_tasks_to_pull = min(
+                    self.config.tasks_per_transaction,
+                    tasks_left,
+                )
+
+            tasks = self.get_next_tasks(
+                number_of_tasks=number_of_tasks_to_pull,
+            )
+            number_of_dequeued_tasks = len(tasks)
+            if number_of_dequeued_tasks == 0:
+                time.sleep(1)
+                time_with_no_tasks += 1
+                if self.config.starvation and time_with_no_tasks >= self.config.starvation.time_with_no_tasks:
+                    self.handle_starvation(
+                        time_with_no_tasks=time_with_no_tasks,
                     )
 
-                tasks = self.get_next_tasks(
-                    number_of_tasks=number_of_tasks_to_pull,
-                )
-                number_of_dequeued_tasks = len(tasks)
-                if number_of_dequeued_tasks == 0:
-                    time.sleep(1)
-                    time_with_no_tasks += 1
-                    if self.config.starvation and time_with_no_tasks >= self.config.starvation.time_with_no_tasks:
-                        self._on_starvation(
-                            time_with_no_tasks=time_with_no_tasks,
-                        )
-
-                    continue
-                else:
-                    time_with_no_tasks = 0
+                continue
+            else:
+                time_with_no_tasks = 0
 
+            try:
+                iterated_tasks = 0
                 for task in tasks:
                     yield task
 
+                    iterated_tasks += 1
+
                 if not run_forever:
                     tasks_left -= number_of_dequeued_tasks
-        except Exception as exception:
-            if tasks:
-                self.apply_async_many(
-                    kwargs_list=[
-                        task['kwargs']
-                        for task in tasks
-                    ],
-                    priority='HIGH',
-                )
+            except Exception as exception:
+                if tasks:
+                    self.apply_async_many(
+                        kwargs_list=[
+                            task['kwargs']
+                            for task in tasks[iterated_tasks + 1:]
+                        ],
+                        priority='HIGH',
+                    )
 
-            raise exception
+                raise exception
 
     def work_loop(
         self,
@@ -260,6 +262,10 @@ class Worker:
             self.executor_obj.execute_tasks(
                 tasks=self.iterate_tasks(),
             )
+        except WorkerRespawn as exception:
+            summary['executor_exception'] = exception
+        except WorkerStop as exception:
+            summary['executor_exception'] = exception
         except Exception as exception:
             self.logger.error(
                 msg=f'execute_tasks has failed: {exception}',
@@ -306,7 +312,17 @@ class Worker:
 
         raise WorkerRequeue()
 
-    def _on_success(
+    def stop(
+        self,
+    ) -> None:
+        raise WorkerStop()
+
+    def respawn(
+        self,
+    ) -> None:
+        raise WorkerRespawn()
+
+    def handle_success(
         self,
         task: typing.Dict[str, typing.Any],
         returned_value: typing.Any,
@@ -324,6 +340,8 @@ class Worker:
                 task=task,
                 returned_value=returned_value,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_success handler has failed: {exception}',
@@ -332,7 +350,7 @@ class Worker:
                 },
             )
 
-    def _on_failure(
+    def handle_failure(
         self,
         task: typing.Dict[str, typing.Any],
         exception: Exception,
@@ -351,17 +369,19 @@ class Worker:
                 exception=exception,
             )
         except WorkerRetry:
-            self._on_retry(
+            self.handle_retry(
                 task=task,
             )
         except WorkerMaxRetries:
-            self._on_max_retries(
+            self.handle_max_retries(
                 task=task,
             )
         except WorkerRequeue:
-            self._on_requeue(
+            self.handle_requeue(
                 task=task,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_failure handler has failed: {exception}',
@@ -370,7 +390,7 @@ class Worker:
                 },
             )
 
-    def _on_timeout(
+    def handle_timeout(
         self,
         task: typing.Dict[str, typing.Any],
     ) -> None:
@@ -387,17 +407,19 @@ class Worker:
                 task=task,
             )
         except WorkerRetry:
-            self._on_retry(
+            self.handle_retry(
                 task=task,
             )
         except WorkerMaxRetries:
-            self._on_max_retries(
+            self.handle_max_retries(
                 task=task,
             )
         except WorkerRequeue:
-            self._on_requeue(
+            self.handle_requeue(
                 task=task,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_timeout handler has failed: {exception}',
@@ -406,7 +428,7 @@ class Worker:
                 },
             )
 
-    def _on_retry(
+    def handle_retry(
         self,
         task: typing.Dict[str, typing.Any],
     ) -> None:
@@ -422,6 +444,8 @@ class Worker:
             self.on_retry(
                 task=task,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_retry handler has failed: {exception}',
@@ -430,7 +454,7 @@ class Worker:
                 },
             )
 
-    def _on_max_retries(
+    def handle_max_retries(
         self,
         task: typing.Dict[str, typing.Any],
     ) -> None:
@@ -446,6 +470,8 @@ class Worker:
             self.on_max_retries(
                 task=task,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_max_retries handler has failed: {exception}',
@@ -454,7 +480,7 @@ class Worker:
                 },
             )
 
-    def _on_requeue(
+    def handle_requeue(
         self,
         task: typing.Dict[str, typing.Any],
     ) -> None:
@@ -470,6 +496,8 @@ class Worker:
             self.on_requeue(
                 task=task,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_requeue handler has failed: {exception}',
@@ -478,7 +506,7 @@ class Worker:
                 },
             )
 
-    def _on_starvation(
+    def handle_starvation(
         self,
         time_with_no_tasks: int,
     ) -> None:
@@ -494,6 +522,8 @@ class Worker:
             self.on_starvation(
                 time_with_no_tasks=time_with_no_tasks,
             )
+        except WorkerInterrupt as exception:
+            raise exception
         except Exception as exception:
             self.logger.error(
                 msg=f'on_starvation handler has failed: {exception}',
@@ -617,3 +647,21 @@ class WorkerMaxRetries(
     WorkerException,
 ):
     pass
+
+
+class WorkerInterrupt(
+    WorkerException,
+):
+    pass
+
+
+class WorkerRespawn(
+    WorkerInterrupt,
+):
+    pass
+
+
+class WorkerStop(
+    WorkerInterrupt,
+):
+    pass
